/*
 *  seen.h
 *
 *  Copyright (C) 2003  Gustavo Picon
 */

#pragma once

#pragma warning(disable:4100)

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "tabolib.h"
#include "ttree.h"


/*
 * #defines
 */

#define MAXSTRINGLEN 900
#define FUNCPARMS    HWND mWnd, HWND aWnd, char *data, char *parms, tbool show, tbool nopause
#define DLLNAME      "seendll"
#define VERSION      "30130"
#define DLLINFO      DLLNAME " " VERSION " (C) Gustavo Picon (http://www.airc.ws)"
#define VERSIONL     DLLNAME " " VERSION
#define MircFunc     extern "C" __declspec(dllexport) int __stdcall


/*
 * Macros
 */

#define r_ok(parms)        { wsprintf(data, "+OK %s", (parms)); return 3; }
#define r_err(code, parms) { wsprintf(data, "-%s %s", (code), (parms)); return 3; }
#define ret(parms)         { lstrcpy(data, (parms)); return 3; }
#define r_s(code, parms)   { wsprintf(data, "+%s %s", (code), (parms)); return 3; }

/*
 * Classes
 */

class SeenNode
{
public:
    SeenNode()
    {}
    SeenNode(char *pnick, char *paddy, char *pchan, char *ptype,
        int ptimespent, char *pmsg, unsigned int pwhen)
    {
        assign(pnick, paddy, pchan, ptype, ptimespent, pmsg, pwhen);
    }
    ~SeenNode()
    {
        if (nick) {
            delete nick;
        }
        if (addy) {
            delete addy;
        }
        if (chan) {
            delete chan;
        }
        if (type) {
            delete type;
        }
        if (msg) {
            delete msg;
        }
    }
    void assign (char *pnick, char *paddy, char *pchan, char *ptype,
        int ptimespent, char *pmsg, unsigned int pwhen)
    {
        nick = new char[lstrlen(pnick) + 1];
        lstrcpy(nick, pnick);
        addy = new char[lstrlen(paddy) + 1];
        lstrcpy(addy, paddy);
        chan = new char[lstrlen(pchan) + 1];
        lstrcpy(chan, pchan);
        type = new char[lstrlen(ptype) + 1];
        lstrcpy(type, ptype);
        msg = new char[lstrlen(pmsg) + 1];
        lstrcpy(msg, pmsg);
        when = pwhen;
        timespent = ptimespent;
    }
    char *nick, *addy, *chan, *type, *msg;
    unsigned int when;
    int timespent;
};


class SeenClass : public CTTree<SeenNode>
{
public:
    static int SeenClass_comparenicks(const SeenNode *i1, const SeenNode *i2);
    static int SeenClass_comparetimes(const SeenNode *i1, const SeenNode *i2);

    SeenClass() : CTTree<SeenNode>(SeenClass_comparenicks)
    {};
    ~SeenClass(void)
    {};

    SeenNode* Add(char *pnick, char *paddy, char *pchan, char *ptype, int ptimespent, char *pmsg, unsigned int pwhen);
    SeenNode* SeenClass::Find(char *nick);
//    void SeenClass::Resort(TreeCompareFunc Func);
    bool SeenClass::Delete(char *nick);
};

typedef struct {
    DWORD mVersion;
    HWND mHwnd;
    BOOL mKeep;
} LOADINFO;

typedef struct seentable_s {
    struct seentable_s *next;
    SeenClass *head;
    char *table;
    hash_t hash;
} *seentable_t;

typedef struct seenstr_s {
    struct seenstr_s *next;
    char *str;
} *seenstr_t;

unsigned int seen_table_cleanup();
void seennodeitemnull (SeenNode *);
